@@ -9,6 +9,8 @@ import dbus
 
 CINNAMON_SOUNDS = "org.cinnamon.sounds"
 CINNAMON_DESKTOP_SOUNDS = "org.cinnamon.desktop.sound"
+MAXIMUM_VOLUME_KEY = "maximum-volume"
+
 DECAY_STEP = .15
 
 EFFECT_LIST = [
@@ -48,6 +50,13 @@ class SoundBox(Gtk.Box):
     def __init__(self, title):
         Gtk.Box.__init__(self)
         self.set_orientation(Gtk.Orientation.VERTICAL)
+        self.set_spacing(5)
+
+        label = Gtk.Label()
+        label.set_markup("<b>%s</b>" % title)
+        label.set_xalign(0.0)
+        self.add(label)
+
         frame = Gtk.Frame()
         frame.set_shadow_type(Gtk.ShadowType.IN)
         frame_style = frame.get_style_context()
@@ -57,25 +66,6 @@ class SoundBox(Gtk.Box):
         main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         frame.add(main_box)
 
-        toolbar = Gtk.Toolbar.new()
-        Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(toolbar), "cs-header")
-        label = Gtk.Label()
-        label.set_markup("<b>%s</b>" % title)
-        title_holder = Gtk.ToolItem()
-        title_holder.add(label)
-        toolbar.add(title_holder)
-        main_box.add(toolbar)
-
-        toolbar_separator = Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
-        main_box.add(toolbar_separator)
-        separator_context = toolbar_separator.get_style_context()
-        frame_color = frame_style.get_border_color(Gtk.StateFlags.NORMAL).to_string()
-        css_provider = Gtk.CssProvider()
-        css_provider.load_from_data(".separator { -GtkWidget-wide-separators: 0; \
-                                                   color: %s;                    \
-                                                }" % frame_color)
-        separator_context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-
         scw = Gtk.ScrolledWindow()
         scw.expand = True
         scw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
@@ -159,10 +149,8 @@ class Slider(SettingsWidget):
         self.slider.add_mark(val, Gtk.PositionType.TOP, "")
 
 class VolumeBar(Slider):
-    def __init__(self, normVolume, maxVolume, title=_("Volume: "), gicon=None, sizeGroup=None):
+    def __init__(self, normVolume, maxPercent, title=_("Volume: "), gicon=None, sizeGroup=None):
         self.normVolume = normVolume
-        self.maxVolume = maxVolume
-        self.maxPercent = 100*maxVolume/normVolume
         self.volume = 0
         self.isMuted = False
         self.baseTitle = title
@@ -172,7 +160,7 @@ class VolumeBar(Slider):
         self.mutedHandlerId = 0
         self.volumeHandlerId = 0
 
-        super(VolumeBar, self).__init__(title, _("Softer"), _("Louder"), 0, self.maxPercent, sizeGroup, 1, 5, 0, gicon)
+        super(VolumeBar, self).__init__(title, _("Softer"), _("Louder"), 0, maxPercent, sizeGroup, 1, 5, 0, gicon)
         self.set_spacing(0)
         self.set_border_width(2)
         self.set_margin_left(23)
@@ -188,7 +176,7 @@ class VolumeBar(Slider):
 
         self.leftBox.pack_start(self.muteSwitch, False, False, 0)
 
-        if maxVolume > normVolume:
+        if maxPercent > 100:
             self.setMark(100)
 
         self.muteSwitchHandlerId = self.muteSwitch.connect("clicked", self.toggleMute)
@@ -481,7 +469,7 @@ class SoundTest(Gtk.Dialog):
             sound = "audio-channel-"+position[1]
 
         session_bus = dbus.SessionBus()
-        sound_dbus = session_bus.get_object("org.cinnamon.SettingsDaemon", "/org/cinnamon/SettingsDaemon/Sound")
+        sound_dbus = session_bus.get_object("org.cinnamon.SettingsDaemon.Sound", "/org/cinnamon/SettingsDaemon/Sound")
         play = sound_dbus.get_dbus_method('PlaySoundWithChannel', 'org.cinnamon.SettingsDaemon.Sound')
         play(0, sound, position[1])
 
@@ -502,22 +490,23 @@ class Module:
     def __init__(self, content_box):
         keywords = _("sound, media, music, speakers, audio")
         self.sidePage = SidePage(_("Sound"), "cs-sound", keywords, content_box, module=self)
+        self.sound_settings = Gio.Settings(CINNAMON_DESKTOP_SOUNDS)
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Sound module"
+            print("Loading Sound module")
 
             self.outputDeviceList = Gtk.ListStore(str, # name
-                                  str, # device
-                                  bool, # active
-                                  int, # id
-                                  GdkPixbuf.Pixbuf) # icon
+                                                  str, # device
+                                                  bool, # active
+                                                  int, # id
+                                                  GdkPixbuf.Pixbuf) # icon
 
             self.inputDeviceList = Gtk.ListStore(str, # name
-                                  str, # device
-                                  bool, # active
-                                  int, # id
-                                  GdkPixbuf.Pixbuf) # icon
+                                                 str, # device
+                                                 bool, # active
+                                                 int, # id
+                                                 GdkPixbuf.Pixbuf) # icon
 
             self.appList = {}
 
@@ -548,7 +537,8 @@ class Module:
         sizeGroup = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
 
         # ouput volume
-        self.outVolume = VolumeBar(self.controller.get_vol_max_norm(), self.controller.get_vol_max_amplified(), sizeGroup=sizeGroup)
+        max_volume = self.sound_settings.get_int(MAXIMUM_VOLUME_KEY)
+        self.outVolume = VolumeBar(self.controller.get_vol_max_norm(), max_volume, sizeGroup=sizeGroup)
         devSettings.add_row(self.outVolume)
 
         # balance
@@ -578,7 +568,7 @@ class Module:
         sizeGroup = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
 
         # input volume
-        self.inVolume = VolumeBar(self.controller.get_vol_max_norm(), self.controller.get_vol_max_amplified(), sizeGroup=sizeGroup)
+        self.inVolume = VolumeBar(self.controller.get_vol_max_norm(), max_volume, sizeGroup=sizeGroup)
         devSettings.add_row(self.inVolume)
 
         # input level
@@ -598,19 +588,19 @@ class Module:
         self.inputStack.add_named(noInputsMessage, "noInputsMessage")
         self.inputStack.show_all()
 
-        ## Effects page
+        ## Sounds page
         page = SettingsPage()
-        self.sidePage.stack.add_titled(page, "effects", _("Sound Effects"))
+        self.sidePage.stack.add_titled(page, "sounds", _("Sounds"))
 
-        effectsVolumeSection = page.add_section(_("Effects Volume"))
-        self.effectsVolume = VolumeBar(self.controller.get_vol_max_norm(), self.controller.get_vol_max_norm())
-        effectsVolumeSection.add_row(self.effectsVolume)
+        soundsVolumeSection = page.add_section(_("Sounds Volume"))
+        self.soundsVolume = VolumeBar(self.controller.get_vol_max_norm(), 100)
+        soundsVolumeSection.add_row(self.soundsVolume)
 
-        effectsSection = SoundBox(_("Effects"))
-        page.pack_start(effectsSection, True, True, 0)
+        soundsSection = SoundBox(_("Sounds"))
+        page.pack_start(soundsSection, True, True, 0)
         sizeGroup = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
         for effect in EFFECT_LIST:
-            effectsSection.add_row(Effect(effect, sizeGroup))
+            soundsSection.add_row(Effect(effect, sizeGroup))
 
         ## Applications page
         page = SettingsPage()
@@ -635,6 +625,25 @@ class Module:
         noAppsMessage.pack_start(box, True, True, 0)
         self.appStack.add_named(noAppsMessage, "noAppsMessage")
 
+        ## Settings page
+        page = SettingsPage()
+        self.sidePage.stack.add_titled(page, "settings", _("Settings"))
+
+        amplificationSection = page.add_section(_("Amplification"))
+        self.maxVolume = Slider(_("Maximum volume: %d") % max_volume + "%", _("Reduced"), _("Amplified"), 1, 150, None, step=1, page=10, value=max_volume, gicon=None, iconName=None)
+        self.maxVolume.adjustment.connect("value-changed", self.onMaxVolumeChanged)
+        self.maxVolume.setMark(100)
+        amplificationSection.add_row(self.maxVolume)
+
+    def onMaxVolumeChanged(self, adjustment):
+        newValue = int(round(adjustment.get_value()))
+        self.sound_settings.set_int(MAXIMUM_VOLUME_KEY, newValue)
+        self.maxVolume.label.set_label(_("Maximum volume: %d") % newValue + "%")
+        self.outVolume.adjustment.set_upper(newValue)
+        self.outVolume.slider.clear_marks()
+        if (newValue > 100):
+            self.outVolume.setMark(100)
+
     def inializeController(self):
         self.controller = Cvc.MixerControl(name = "cinnamon")
         self.controller.connect("state-changed", self.setChannelMap)
@@ -671,19 +680,22 @@ class Module:
         id = getattr(self.controller, "lookup_"+type+"_id")(newDevice)
         if id != None and id != getattr(self, type+"Id"):
             getattr(self.controller, "change_"+type)(id)
+            self.profile.setDevice(id)
 
     def deviceAdded(self, c, id, type):
         device = getattr(self.controller, "lookup_"+type+"_id")(id)
 
         iconTheme = Gtk.IconTheme.get_default()
         gicon = device.get_gicon()
+        iconName = device.get_icon_name()
         icon = None
         if gicon is not None:
             lookup = iconTheme.lookup_by_gicon(gicon, 32, 0)
             if lookup is not None:
                 icon = lookup.load_icon()
-        if icon is not None:
-            if ("bluetooth" in device.get_icon_name()):
+
+        if icon is None:
+            if (iconName is not None and "bluetooth" in iconName):
                 icon = iconTheme.load_icon("bluetooth", 32, 0)
             else:
                 icon = iconTheme.load_icon("audio-card", 32, 0)
@@ -758,13 +770,34 @@ class Module:
         stream = self.controller.lookup_stream_id(id)
 
         if stream in self.controller.get_sink_inputs():
-            self.appList[id] = VolumeBar(self.controller.get_vol_max_norm(), self.controller.get_vol_max_norm(), stream.props.name + ": ", stream.get_gicon())
+            name = stream.props.name
+
+            # FIXME: We use to filter out by PA_PROP_APPLICATION_ID.  But
+            # most streams report this as null now... why??
+            if name in ("speech-dispatcher", "libcanberra"):
+                # speech-dispatcher: orca/speechd/spd-say
+                # libcanberra: cinnamon effects, test sounds
+                return
+
+            if id in self.appList.keys():
+                # Don't add an input more than once
+                return
+
+            if name == None:
+                name = _("Unknown")
+
+            label = "%s: " % name
+
+            self.appList[id] = VolumeBar(self.controller.get_vol_max_norm(),
+                                         100,
+                                         label,
+                                         stream.get_gicon())
             self.appList[id].setStream(stream)
             self.appSettings.add_row(self.appList[id])
             self.appSettings.list_box.invalidate_headers()
             self.appSettings.show_all()
         elif stream == self.controller.get_event_sink_input():
-            self.effectsVolume.setStream(stream)
+            self.soundsVolume.setStream(stream)
 
         self.checkAppState()
 
