@@ -1,15 +1,30 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
 from gi.repository.Gtk import SizeGroup, SizeGroupMode
 
 from GSettingsWidgets import *
-from CinnamonGtkSettings import GtkSettingsSwitch
-from ExtensionCore import ExtensionSidePage
+from CinnamonGtkSettings import CssRange, CssOverrideSwitch, GtkSettingsSwitch, PreviewWidget, Gtk2ScrollbarSizeEditor
+from SettingsWidgets import LabelRow
+from ExtensionCore import DownloadSpicesPage
+from Spices import Spice_Harvester
 
 import glob
 
 ICON_SIZE = 48
 
+# Gtk and Cinnamon check folders in order of precedence.  These lists match the
+# order.  It doesn't really matter here, since we're only looking for names,
+# but it's helpful to be aware of it.
+
+ICON_FOLDERS = [
+    os.path.join(GLib.get_user_data_dir(), "icons"),
+    os.path.join(GLib.get_home_dir(), ".icons")
+] + [os.path.join(datadir, "icons") for datadir in GLib.get_system_data_dirs()]
+
+THEME_FOLDERS = [
+    os.path.join(GLib.get_user_data_dir(), "themes"),
+    os.path.join(GLib.get_home_dir(), ".themes")
+] + [os.path.join(datadir, "themes") for datadir in GLib.get_system_data_dirs()]
 
 class Module:
     comment = _("Manage themes to change how your desktop looks")
@@ -19,12 +34,15 @@ class Module:
     def __init__(self, content_box):
         self.keywords = _("themes, style")
         self.icon = "cs-themes"
+        self.window = None
         sidePage = SidePage(_("Themes"), self.icon, self.keywords, content_box, module=self)
         self.sidePage = sidePage
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Themes module"
+            print("Loading Themes module")
+
+            self.spices = Spice_Harvester('theme', self.window)
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -57,14 +75,11 @@ class Module:
             settings.add_row(widget)
 
             widget = self.make_group(_("Desktop"), self.cinnamon_chooser)
-            center_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-            button = Gtk.LinkButton()
-            button.set_label(_("Add/remove desktop themes..."))
-            button.connect("activate-link", self.add_remove_cinnamon_themes)
-            center_box.pack_end(button, False, False, 0)
-            widget.pack_start(center_box, False, False, 0)
             settings.add_row(widget)
 
+            page = DownloadSpicesPage(self, 'theme', self.spices, self.window)
+            self.sidePage.stack.add_titled(page, 'download', _("Add/Remove"))
+
             page = SettingsPage()
             self.sidePage.stack.add_titled(page, "options", _("Settings"))
 
@@ -76,13 +91,37 @@ class Module:
             widget = GSettingsSwitch(_("Show icons on buttons"), "org.cinnamon.settings-daemon.plugins.xsettings", "buttons-have-icons")
             settings.add_row(widget)
 
-            dark_text = _("Use a dark theme variant when available in certain applications")
-            dark_italic = _("(Applications must be restarted for this change to take effect)")
+            settings = page.add_section(_("Scrollbar behavior"))
 
-            widget = GtkSettingsSwitch("%s\n<i><small>%s</small></i>" % (dark_text, dark_italic),
-                                       "gtk-application-prefer-dark-theme")
+            switch = GtkSettingsSwitch(_("Warp sliders when clicking in a trough"), "gtk-primary-button-warps-slider")
+            settings.add_row(switch)
+
+            widget = GSettingsSwitch(_("Use overlay scroll bars"), "org.cinnamon.desktop.interface", "gtk-overlay-scrollbars")
             settings.add_row(widget)
 
+            self.gtk2_scrollbar_editor = Gtk2ScrollbarSizeEditor(widget.get_scale_factor())
+
+            switch = CssOverrideSwitch(_("Override the current theme's scrollbar width"))
+            settings.add_row(switch)
+            self.scrollbar_switch = switch.content_widget
+
+            widget = CssRange(_("Scrollbar width"), "scrollbar slider", ["min-width", "min-height"], 5, 100, "px", None, switch)
+            settings.add_reveal_row(widget)
+            widget.sync_initial_switch_state()
+            self.scrollbar_css_range = widget.content_widget
+
+            switch.content_widget.connect("notify::active", self.on_css_override_active_changed)
+            widget.content_widget.connect("value-changed", self.on_range_slider_value_changed)
+
+            self.on_css_override_active_changed(switch)
+
+            widget = PreviewWidget()
+            settings.add_row(widget)
+
+            label_widget = LabelRow(_(
+"""Changes will take effect the next time you log in and may not affect all applications."""))
+            settings.add_row(label_widget)
+
             self.builder = self.sidePage.builder
 
             for path in [os.path.expanduser("~/.themes"), os.path.expanduser("~/.icons")]:
@@ -95,12 +134,26 @@ class Module:
             for path in [os.path.expanduser("~/.themes"), "/usr/share/themes", os.path.expanduser("~/.icons"), "/usr/share/icons"]:
                 if os.path.exists(path):
                     file_obj = Gio.File.new_for_path(path)
-                    file_monitor = file_obj.monitor_directory(Gio.FileMonitorFlags.SEND_MOVED, None)
-                    file_monitor.connect("changed", self.on_file_changed)
-                    self.monitors.append(file_monitor)
+                    try:
+                        file_monitor = file_obj.monitor_directory(Gio.FileMonitorFlags.SEND_MOVED, None)
+                        file_monitor.connect("changed", self.on_file_changed)
+                        self.monitors.append(file_monitor)
+                    except Exception as e:
+                        # File monitors can fail when the OS runs out of file handles
+                        print(e)
 
             self.refresh()
 
+    def on_css_override_active_changed(self, switch, pspec=None, data=None):
+        if self.scrollbar_switch.get_active():
+            self.gtk2_scrollbar_editor.set_size(self.scrollbar_css_range.get_value())
+        else:
+            self.gtk2_scrollbar_editor.set_size(0)
+
+    def on_range_slider_value_changed(self, widget, data=None):
+        if self.scrollbar_switch.get_active():
+            self.gtk2_scrollbar_editor.set_size(widget.get_value())
+
     def on_file_changed(self, file, other, event, data):
         self.refresh()
 
@@ -122,7 +175,6 @@ class Module:
             callback = chooser[3]
             payload = (chooser_obj, path_suffix, themes, callback)
             self.refresh_chooser(payload)
-            # thread.start_new_thread(self.refresh_chooser, (payload,))
 
     def refresh_chooser(self, payload):
         (chooser, path_suffix, themes, callback) = payload
@@ -148,8 +200,8 @@ class Module:
                 theme_path = theme[1]
                 try:
                     for path in ["%s/%s/%s/thumbnail.png" % (theme_path, theme_name, path_suffix),
-                             "/usr/share/cinnamon/thumbnails/%s/%s.png" % (path_suffix, theme_name),
-                             "/usr/share/cinnamon/thumbnails/%s/unknown.png" % path_suffix]:
+                                 "/usr/share/cinnamon/thumbnails/%s/%s.png" % (path_suffix, theme_name),
+                                 "/usr/share/cinnamon/thumbnails/%s/unknown.png" % path_suffix]:
                         if os.path.exists(path):
                             chooser.add_picture(path, callback, title=theme_name, id=theme_name)
                             break
@@ -157,7 +209,6 @@ class Module:
                     chooser.add_picture("/usr/share/cinnamon/thumbnails/%s/unknown.png" % path_suffix, callback, title=theme_name, id=theme_name)
                 GLib.timeout_add(5, self.increment_progress, (chooser, inc))
         GLib.timeout_add(500, self.hide_progress, chooser)
-        # thread.exit()
 
     def increment_progress(self, payload):
         (chooser, inc) = payload
@@ -168,7 +219,7 @@ class Module:
         chooser.reset_loading_progress()
 
     def _setParentRef(self, window):
-        pass
+        self.window = window
 
     def make_group(self, group_label, widget, add_widget_to_size_group=True):
         self.size_groups = getattr(self, "size_groups", [Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL) for x in range(2)])
@@ -210,27 +261,13 @@ class Module:
                 chooser.set_picture_from_file("/usr/share/cinnamon/thumbnails/%s/unknown.png" % path_suffix)
         return chooser
 
-    def add_remove_cinnamon_themes(self, widget):
-        window = Gtk.Window()
-        box = Gtk.VBox()
-        window.add(box)
-        window.set_title(_("Desktop themes"))
-        window.set_default_size(720, 480)
-        window.set_border_width(6)
-        window.set_position(Gtk.WindowPosition.CENTER)
-        page = ExtensionSidePage(self.name, self.icon, self.keywords, box, "theme", None)
-        page.load(window=window)
-        box.pack_start(page.vbox, True, True, 6)
-        window.show_all()
-        return True
-
     def _on_icon_theme_selected(self, path, theme):
         try:
             self.settings.set_string("icon-theme", theme)
             self.icon_chooser.set_button_label(theme)
             self.icon_chooser.set_tooltip_text(theme)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
         return True
 
     def _on_metacity_theme_selected(self, path, theme):
@@ -238,8 +275,8 @@ class Module:
             self.wm_settings.set_string("theme", theme)
             self.metacity_chooser.set_button_label(theme)
             self.metacity_chooser.set_tooltip_text(theme)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
         return True
 
     def _on_gtk_theme_selected(self, path, theme):
@@ -247,8 +284,8 @@ class Module:
             self.settings.set_string("gtk-theme", theme)
             self.theme_chooser.set_button_label(theme)
             self.theme_chooser.set_tooltip_text(theme)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
         return True
 
     def _on_cursor_theme_selected(self, path, theme):
@@ -256,8 +293,10 @@ class Module:
             self.settings.set_string("cursor-theme", theme)
             self.cursor_chooser.set_button_label(theme)
             self.cursor_chooser.set_tooltip_text(theme)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
+
+        self.update_cursor_theme_link(path, theme)
         return True
 
     def _on_cinnamon_theme_selected(self, path, theme):
@@ -265,15 +304,15 @@ class Module:
             self.cinnamon_settings.set_string("name", theme)
             self.cinnamon_chooser.set_button_label(theme)
             self.cinnamon_chooser.set_tooltip_text(theme)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
         return True
 
     def _load_gtk_themes(self):
         """ Only shows themes that have variations for gtk+-3 and gtk+-2 """
-        dirs = ("/usr/share/themes", os.path.join(os.path.expanduser("~"), ".themes"))
+        dirs = THEME_FOLDERS
         valid = walk_directories(dirs, self.filter_func_gtk_dir, return_directories=True)
-        valid.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
+        valid.sort(key=lambda a: a[0].lower())
         res = []
         for i in valid:
             for j in res:
@@ -296,7 +335,7 @@ class Module:
         return False
 
     def _load_icon_themes(self):
-        dirs = ("/usr/share/icons", os.path.join(os.path.expanduser("~"), ".icons"))
+        dirs = ICON_FOLDERS
         walked = walk_directories(dirs, lambda d: os.path.isdir(d), return_directories=True)
         valid = []
         for directory in walked:
@@ -310,7 +349,7 @@ class Module:
                 except Exception as e:
                     print (e)
 
-        valid.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
+        valid.sort(key=lambda a: a[0].lower())
         res = []
         for i in valid:
             for j in res:
@@ -323,9 +362,9 @@ class Module:
         return res
 
     def _load_cursor_themes(self):
-        dirs = ("/usr/share/icons", os.path.join(os.path.expanduser("~"), ".icons"))
+        dirs = ICON_FOLDERS
         valid = walk_directories(dirs, lambda d: os.path.isdir(d) and os.path.exists(os.path.join(d, "cursors")), return_directories=True)
-        valid.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
+        valid.sort(key=lambda a: a[0].lower())
         res = []
         for i in valid:
             for j in res:
@@ -338,9 +377,9 @@ class Module:
         return res
 
     def _load_metacity_themes(self):
-        dirs = ("/usr/share/themes", os.path.join(os.path.expanduser("~"), ".themes"))
-        valid = walk_directories(dirs, lambda d: os.path.exists(os.path.join(d, "metacity-1")), return_directories=True)
-        valid.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
+        dirs = THEME_FOLDERS
+        valid = walk_directories(dirs, lambda d: os.path.exists(os.path.join(d, "metacity-1/metacity-theme-3.xml")), return_directories=True)
+        valid.sort(key=lambda a: a[0].lower())
         res = []
         for i in valid:
             for j in res:
@@ -353,9 +392,9 @@ class Module:
         return res
 
     def _load_cinnamon_themes(self):
-        dirs = ("/usr/share/themes", os.path.join(os.path.expanduser("~"), ".themes"))
+        dirs = THEME_FOLDERS
         valid = walk_directories(dirs, lambda d: os.path.exists(os.path.join(d, "cinnamon")), return_directories=True)
-        valid.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
+        valid.sort(key=lambda a: a[0].lower())
         res = []
         for i in valid:
             for j in res:
@@ -366,3 +405,20 @@ class Module:
                         res.remove(j)
             res.append((i[0], i[1]))
         return res
+
+    def update_cursor_theme_link(self, path, name):
+        default_dir = os.path.join(os.path.expanduser("~"), ".icons", "default")
+        index_path = os.path.join(default_dir, "index.theme")
+
+        try:
+            os.makedirs(default_dir)
+        except os.error as e:
+            pass
+
+        if os.path.exists(index_path):
+            os.unlink(index_path)
+
+        contents = "[icon theme]\nInherits=%s\n" % name
+
+        with open(index_path, "w") as f:
+            f.write(contents)
